# -*- coding: utf-8 -*-

import os


def mkGrids(w, h):
    grids = []
    for y in range(h):
        grids.append([])
        for x in range(w):
            grids[y].append(1)

    return grids


def mkLine(line_mask, n):
    return [1 if i & n else 0 for i in line_mask]


def mkFirstLines(w):
    u"使用 yield ，而不是一次生生成整个列表，防止 w 较大时内存不够"
    # lines = []
    line_mask = [1 << (w - i - 1) for i in range(w)]
    for i in xrange(2 ** w):
        # lines.append(mkLine(line_mask, i))
        yield mkLine(line_mask, i)


# return lines


def light2(grids, x, y, w, h):
    if x >= 0 and x < w and y >= 0 and y < h:
        grids[y][x] = 1 if grids[y][x] == 0 else 0


def light(grids, x, y, w, h):
    light2(grids, x, y, w, h)
    light2(grids, x + 1, y, w, h)
    light2(grids, x - 1, y, w, h)
    light2(grids, x, y + 1, w, h)
    light2(grids, x, y - 1, w, h)


#	print(x, y)
#	show(grids)


def show(grids):
    print "-" * 10
    for line in grids:
        lns = ""
        for i in line:
            lns += "%d " % i
        print lns
    print


def chkValid(w, h, line):
    u"返回完成所需的步数，如不能完成，则返回 0"
    data = mkGrids(w, h)
    steps = 0
    #	print("-" * 50)
    #	print("check:", line)

    # first line
    for x, v in enumerate(line):
        if v == 1:
            light(data, x, 0, w, h)
            steps += 1

    for y in range(1, h):
        previous_line = data[y - 1]
        for x, v in enumerate(previous_line):
            if v == 1:
                light(data, x, y, w, h)
                steps += 1

    # 检查最后一行是否全被熄灭
    the_last_line = data[-1]
    return 0 if sum(the_last_line) != 0 else steps


def solve(w, h):
    u"求解 w * h 的布局"
    print ("%d x %d:" % (w, h)),
    solutions = []
    for line in mkFirstLines(w):
        steps = chkValid(w, h, line)
        if steps > 0:
            solutions.append([steps, line])
    count = len(solutions)
    print "%d solution%s" % (count, "s" if count > 1 else "")

    solutions.sort(key=lambda x: x[0])
    if count > 0:
        print solutions[0]  # 打印出一种解法

    outf = open(os.path.join("solutions", "%dx%d.json" % (w, h)), "w")
    outf.write(repr(solutions))
    outf.close()


def main(n=10):
    for i in xrange(1, n + 1):
        solve(i, i)


if __name__ == "__main__":
    main(50)
